%********************************************************************
%                                                                   * 
%                       Data Mining Examples                        *
%                                                                   * 
%********************************************************************

%********************************************************************
%     CLASSIFICATION PROBLEM - PREDICT CLASS LABELS (DISCRETE)      *
%                     A Neural Network Example                      *
%********************************************************************

clear

% Pima Indians Diabetes Database
%   ftp://ftp.ics.uci.edu/pub/machine-learning-databases/pima-indians-diabetes/
% Description in file "pima.names", data in file "pima.data"

% The goal: to classify patients into 2 classes, i.e. whether a patient shows signs of diabetes, based on 8 features

% load the dataset
pima = load('pima.data');
[nr, nc] = size(pima)
nf = nc - 1;      % number of features

% Transform the target representation: class label '0/1' to '2/1'
pima(:, nc) = pima(:,nc)+1;

num_class = 2;    % 2 classes

% Split the data into training, validation and test subsets
% Leave 30% of data in test set
[ptrval, ptest] = divideset(pima, 30);     
% Assign 30% of tr_val to validation and 70% to training set
[ptrain, pval]   = divideset(ptrval, 30);    

% normalize the training set (not including the target variable!)
[meanv, stdv, ptrain(:, 1 : nf)] = normalize(ptrain(:, 1 : nf), [], []);
% normalize the validation set using 'meanv' and 'stdv' calculated from the training set
[t1, t2, pval(:, 1 : nf)] = normalize(pval(:, 1 : nf), meanv, stdv);

% Transform the target representation: class '1' -> [1 0], class '2' -> [0 1]  
ptrain_c = class_matrix_gen1(ptrain(:, nc), 0.9, num_class);
pval_c = class_matrix_gen1(pval(:, nc), 0.9, num_class);

% INITIALIZE NEURAL NETWORK
%   we use a feed-forward neural network with one hidden layer, 'tansig' sigmoid activation functions in the hidden layer,
%   'logsig' sigmoid activation in the output layer. 
num_hidd = 5;   % 5 neurons in the hidden layer

% SELECT TRAINING ALGORITHM
method = 2;
switch method
  case 1,   train_alg = 'traingd'    % Standard Back Propagation 
  case 2,	train_alg = 'trainrp'    % Resilient - Back propagation
  case 3,	train_alg = 'traingdx'   % Variable LEarning Rate
  case 4,	train_alg = 'traincgb'   % Powell-Beale Conjugate Gradient
  case 5,	train_alg = 'trainlm'    % Levenberg-Marquardt                 <--
  case 6,	train_alg = 'trainbfg'   % Quasi-Newton Algorithm
  case 7,	train_alg = 'trainscg'   % Scaled Conjugate Gradient
  otherwise, train_alg = 'traingd'
end

% OBSERVE THAT WE HAVE NOW num_class OUTPUT NODES
pima_net = newff([min(ptrain(:, 1 : nf)); max(ptrain(:, 1 : nf))]', [num_hidd num_class], {'tansig', 'logsig'}, train_alg)

% Define other important parameters for neural network training
% If ommit learning rate, Matlab automatically determines the learnig rate
% which is better in many cases
pima_net.trainParam.lr = 0.1;      % Choose learning rate
pima_net.trainParam.epochs = 250;  % maximum number of epochs
pima_net.trainParam.show = 10;     % show results each 10 epochs
pima_net.trainParam.max_fail = 5;  % stop if the error on validation set does not improve over 5 iterations

% PERFORM TRAINING
% Reshape training, validation and test set to fit neural network training function
P = ptrain(:, 1 : nf)';
T = ptrain_c';
VV.P = pval(:, 1 : nf)';
VV.T = pval_c';
% Actual training
[pima_net, pima_net_train] = train(pima_net, P, T, [], [], VV);

% check the predictions on test set
ptest_c = ptest(:, nf + 1);
[t1, t2, ptest(:, 1 : nf)] = normalize(ptest(:, 1 : nf), meanv, stdv);
pima_pred = sim(pima_net, ptest(:, 1 : nf)');
pima_decided = class_decision(pima_pred, 0);

% evaluate accuracy of prediction
TN = sum(ptest_c == 1 & pima_decided == 1); % true class 1, predicted class 1
FP = sum(ptest_c == 1 & pima_decided == 2); % true class 1, predicted class 2
FN = sum(ptest_c == 2 & pima_decided == 1); % true class 2, predicted class 1
TP = sum(ptest_c == 2 & pima_decided == 2); % true class 2, predicted class 2

accuracy1 = (TN + TP) / (TP+TN+FP+FN)

% maybe, a better way to calculate accuracy is for both classes
sensitivity = TP / (TP+FN) % accuracy on class 1
specificity = TN / (TN+FP) % accuracy on class 0
accuracy2 = (sensitivity + specificity) / 2

disp 'done'
