%********************************************************************
%                                                                   *
%                       Data Mining Examples                        *
%                                                                   *
%********************************************************************

%********************************************************************
%    REGRESSION PROBLEM - PREDICT REAL-VALUE OUTPUT (CONTINUOUS)    *
%                       A Neural Network Example                    *
%********************************************************************

clear

% The "Boston Housing" dataset from UCI Machine Learning Repository
%    ftp://ftp.ics.uci.edu/pub/machine-learning-databases/housing/
% Description in file "housing.names", data in file "housing.data"

% The goal: to predict the median value of owner-occupied homes based on 13 attributes

% Load the data (13 features, 1 target variable)
data = load('housing.data','-ascii');

[nr, nc] = size(data)
nf = nc - 1;           % number of features/attributes

% 1. FIRST DO SOME VISUAL INSPECTION OF THE DATA

% Look at the histogram of the first feature and target variable
figure(1);
hist(data(:, 1), 15);  % continuous feature
hist(data(:, 4), 15);  % binary feature
hist(data(:, 14), 15); % target

% caclulate correlation coefficient between columns 4 and 7
corrcoef(data(:, [4 7]))

% visualize linear correlation between 3 features and a target
plotmatrix(data(:, [4 6 13 14]))

% 2. NORMALIZE THE DATA - we want all the features/attributes and the target variable to have the same means (0) and variances (1)
[meanv, stdv, data] = normalize(data, [], []);

% 3. SPLIT THE DATASET
% We should randomly split the whole dataset into 3 disjoint subsets: training, validation and test.
% Reserve 30% of data as test set, for estimating the accuracy
[trval, test] = divideset(data, 30);
% Assign 30% of trval to validation and 70% to training set
[tdata, vdata] = divideset(trval, 30);

% 4. INITIALIZE NEURAL NETWORK
% we need choose the architecture of the neural network. For this example, we will use a feed-forward
% neural network with one hidden layer, sigmoid activation functions in the hidden layer, and linear
% activation function in the output layer.

num_hidd = 10;   % 10 neurons in the hidden layer

% Select a neural network training algorithm (for adjusting the weights)

methodselection = [1 2 5 6];
for method_index = 1:length(methodselection)
    method = methodselection(method_index);
    switch method
      case 1,    train_alg = 'traingd'    % Standard Back Propagation
      case 2,    train_alg = 'trainrp'    % Resilient - Back propagation
      case 3,    train_alg = 'traingdx'   % Variable Learning Rate
      case 4,    train_alg = 'traincgb'   % Powell-Beale Conjugate Gradient
      case 5,    train_alg = 'trainlm'    % Levenberg-Marquardt                 <-- selected
      case 6,    train_alg = 'trainbfg'   % Quasi-Newton Algorithm
      case 7,    train_alg = 'trainscg'   % Scaled Conjugate Gradient
      otherwise, train_alg = 'traingd'
    end

    % Create the neural network object
    net = newff([min(tdata(:, 1 : nf)); max(tdata(:, 1 : nf))]', ...  % the range of each attribute
                [num_hidd 1],  ...                                                        % # of neurons in the hidden and output layers, respectively
                {'logsig', 'purelin'}, ...                                                % activation functions for neurons in the hidden and output layers, respectively
                train_alg)                                                                % training algorithm

    % Define other important parameters for neural network training. If ommit learning rate, Matlab automatically determines
    % the learnig rate, which might be better in many cases.
    % Note that there are other parameters specific for more complex
    % training algorithms that are not set here, so this does not provide a
    % good comparison of performance of various algorithms. 
    net.trainParam.lr = 0.1;      % Choose learning rate
    net.trainParam.epochs = 300;  % maximum number of epochs
    net.trainParam.show = 10;     % show results each 10 epochs
    net.trainParam.max_fail = 5;  % stop if the error on validation set does not improve over 5 iterations

    % 5. PERFORM TRAINING
    % Reshape training, validation and test sets to fit the neural network training function
    % Matlab's neural network functions store datapoints in columns, not rows,
    % therefore we have to transpose the data.
    P = tdata(:, 1 : nf)'; 
    T = tdata(:, nf + 1)';
    VV.P = vdata(:, 1 : nf)';
    VV.T = vdata(:, nf + 1)';
    TV.P = test(:, 1 : nf)';
    TV.T = test(:, nf + 1)';
    % The actual training
    [net, tc] = train(net, P, T, [], [], VV, TV);

    pause();
    close
    
    % 6. MAKE PREDICTION ON THE TEST SET AND ESTIAMTE ACCURACY
    % prediction
    prediction = sim(net, test(:, 1 : nf)')';

    pred_error = prediction - test(:, nf + 1);

    % % plot the histogram of errors
    % hist(pred_error);

    % plot real value vs predicted value
    figure(2);
    subplot(2,2,method_index);
    plot(test(:,nf+1),prediction,'.');
    valuerange = minmax([prediction'  test(:,nf+1)']);
    axis([valuerange valuerange]);
    axis square
    hold on
    plot(valuerange, valuerange,'r:');
    hold off

    % Calculate MEAN Squared Error (MSE)
    mse = (pred_error' * pred_error) / length(pred_error)
    % Calculate R-square (PREDICTION ACCURACY)
    R2 = 1 - mse / var(test(:, 14))
    title(sprintf('%s, R2 = %.3f', train_alg, R2));

    % add some labels
    xlabel('true value')
    ylabel('predicted value')

    pause
end

disp 'done'

